from tests.integration.conftest import (
    ATTENTION_PAPER_DOI,
    ATTENTION_PAPER_TITLE,
    ATTENTION_PAPER_YEAR,
    BERT_PAPER_DOI,
    BERT_PAPER_TITLE,
    BERT_PAPER_YEAR,
    QueryExpectation,
    assert_paper_satisfies,
    assert_search_result_valid,
    assert_valid_paper,
    requires_scopus,
    requires_unpaywall,
)

__all__ = [
    "ATTENTION_PAPER_DOI",
    "ATTENTION_PAPER_TITLE",
    "ATTENTION_PAPER_YEAR",
    "BERT_PAPER_DOI",
    "BERT_PAPER_TITLE",
    "BERT_PAPER_YEAR",
    "QueryExpectation",
    "assert_paper_satisfies",
    "assert_search_result_valid",
    "assert_valid_paper",
    "requires_scopus",
    "requires_unpaywall",
]
